/*******************************************************************************

    COFRINHO DE COMBINAO USANDO POTNCIOMETRO LINEAR DE 10K

    C/ INDICAO POR DISPLAYS DE 7 SEGUIMENTOS


    C/ PIC16F628A


    AUTOR: CLAUDIO LARIOS

    DATA:25/12/2023




    MODO PROGRAMAO: MANTENHA BOTO 'PROG' APERTADO E LIGUE A
         ALIMENTAO. LED 'PROG' ACENDER.
         ENTRE COM A SENHA:
         VIRE O KNOB AT A POSIO DESEJADA. VERIFIQUE NO DISPLAY O VALOR.
         APERTE O BOTO 'SW_ENTER'. O LED PISCA PARA INDICAR A INSERO.
         GIRE O KNOB AT A PRXIMA POSIO DESEJADA E REPITA O PROCESSO
         COM TODA A SENHA. AO ENTRAR A LTIMA POSIO
         O LED 'PROG' APAGAR, INDICANDO SUCESSO NA GRAVAO.

    USO: REPITA A SENHA GRAVADA DE 5 DGITOS.


    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS

*******************************************************************************/
#include <16F628A.h>
#use delay(clock=4000000)//USE CRISTAL EXTERNO DE 4 MHZ
#fuses NOWDT,HS, NOPUT, NOPROTECT, BROWNOUT, NOMCLR, NOLVP, NOCPD



//bits
#bit bprog    =  0x05.5 // pino 4
#bit btn      =  0x05.4 // pino 3
#bit ledprog  =  0x05.0 // pino 17
#bit led      =  0x05.1 // pino 18
#bit out      =  0x05.2 // pino 1
#bit rc       =  0x05.3 // pino 2
#bit rc_dir   =  0x85.3 // pino 2 direo
#bit sel      =  0x06.7//  pino 13 seleo de display

//bytes
#byte  trisa = 0x85
#byte  trisb = 0x86
#byte  porta   = 0x05
#byte  portb   = 0x06
#byte  option_reg = 0x81
#byte intcon = 0x0b

#define VALOR_ESTAB 50

#define VALOR_PARA_ZERAR 6// elimina leitura at este valor com o potencimetro no mnimo
#define TEMPO_ENTRAR_SENHA 50000 // carga para o tempo de entrar a senha depois de acionado boto
#define TEMPO_PROG_MAX 25000 // carga para o tempo de time out para a programao
#define QUANT_DIGITOS  5  //quantidade de digitos na senha = 5
#define TEMPO_ACIONAMENTO 5000 //tempo que fechadura eltrica  acionada (em ms)

//constantes para converso 7 seguimentos

//                         PGFEDCBA   SEGUIMENTOS
//                   PORTB 76543210
int8 const NUM[] = {     0B01000000,//0
                         0B01111001,//1
                         0B00100100,//2
                         0B00110000,//3
                         0B00011001,//4
                         0B00010010,//5
                         0B00000010,//6
                         0B01111000,//7
                         0B00000000,//8
                         0B00010000 //9
                                        };


//==============================================================================

//variveis globais
unsigned int1 flagat=0,flagv=0,flagok=0,flagrcok=0;
unsigned int8 val[QUANT_DIGITOS],contrc=0,oldb=0,tmp=0,tmp1=0;
unsigned int8 g,h,ptn1=0,contd=0,dig0a=8,dig1a=8,dig0=8,dig1=8;
unsigned int32 timer5=0,timer6=0;

/*******************************************************************************
          SUB-ROTINAS
*******************************************************************************/
#INT_TIMER0
void INT_TIMER0_ISR(){

 //calcula posio do potencimetro pelo tempo de carga dos capacitores
  if(flagrcok==0){
    rc_dir=1;//entrada
    while(rc==0)contrc++;//aguarda pino 2 (entrada) ficar 1 lgico
    rc=0;//zera sada
    rc_dir=0;//torna pino 2 sada
    flagrcok=1;//sinaliza com 1 no flag que existe uma leitura do potencimetro
  }
  contd++;//contador auxiliar para multiplexao dos displays de 7 seguimento
  if(contd>=8){//mostra no display a cada 8 interrupes 
   contd=0;
   portb=0xff; //desliga todos os seguimentos
   if(flagok){//se tem alterao no valor repassa para os registradores do displays
     dig0a=dig0;
     dig1a=dig1;
     flagok=0;//reseta flag
   }

   if(flagv){//verifica a vez de quem vai ser mostrado: dezena ou unidade.
     portb= NUM[dig1a]|0x80;
     flagv=0;//inverte flag
   }
   else{
    portb= NUM[dig0a];
    flagv=1;//inverte flag
   }
  }//if(contd>=8){
}//void INT_TIMER0_ISR(){


/*******************************************************************************
          le_adc()
*******************************************************************************/
//rotina de leitura do potencimetro e acondicionamento de valores
//mtodo de carga e descarga de capacitor
unsigned int8 le_adc(){
  unsigned int8 b,c;

   while(flagrcok==0);
   b=contrc;//pega valor da contagem do tempo rc
   contrc=0;//reseta contador
   flagrcok=0;//prepara para nova contagem na isr do timer 0

   flagok=0;//inibe troca de valores na isr do timer 0

   b=(b + oldb )/2;//mdia com leitura anterior
   
   //------------------------------------------------
   //------------filtro anti-flickering--------------------
   //Filtro digital para estabilizao da leitura no
   //display de 7 seguimentos
   
   if(b==oldb){tmp=0;tmp1=0;}
   
   else {if(b>oldb){
     tmp++;
     tmp1=0;
     if(tmp>=VALOR_ESTAB){oldb=b; tmp=0;}else {b=oldb;}
    }
     else {if(b<oldb){
       tmp1++;
       tmp=0;
       if(tmp1>=VALOR_ESTAB){oldb=b;tmp1=0;}else{b=oldb;}   
     }
    }
   }
   //--------------------------------------------------------
   c=b;
   oldb=b;//salva valor para prxima comparao
   //Ajusta a faixa a ser mostrada no display  de 0 a 99
   if(c>=VALOR_PARA_ZERAR)//despreza valor no mnimo do potencimetro e zera
     c=c-VALOR_PARA_ZERAR;
   else c=0;
     
   if(c>=100)c=99;//limita escala

   b=c;
   dig1=0;
   //converte para decimal dgito das dezenas
   while (c>=10){c=c-10; dig1++;}
   //obtm a unidade com o resto
   dig0=c;

  
   flagok=1;//autoriza troca de valores para apresentao na isr do timer 0
   return(b);//retorna valor
}



/*******************************************************************************
          prog_mode()
*******************************************************************************/
//Modo de programao de senha pelo potencimetro e boto
void prog_mode(){

  if(!bprog){
    delay_ms(50);
    if(!bprog){
      ledprog=1;//acende led
      timer5=TEMPO_PROG_MAX;
    }
  }
}
/*******************************************************************************
          time_out_prog()
*******************************************************************************/
//sai do modo de programao por tempo
void time_out_prog(void){
  if(ledprog){
   timer5--;
   if(timer5==0)ledprog=0;
  }
}
/*******************************************************************************
          time_out_abert()
*******************************************************************************/
//sai do modo de insero de senha por tempo
void time_out_insercao(void){
  if(led){
   timer6--;
   if(timer6==0){led=0; ptn1=0;delay_ms(500); timer6=1;}//xxxxx
  }
}




/*******************************************************************************
          ROTINA MAIN
*******************************************************************************/

void main() {


   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);//desliga timer 1
   setup_comparator(NC_NC_NC_NC);//desliga comparadores
   setup_vref(FALSE);//desliga vref
   trisa=0b11111000;//ajusta entradas e sadas
   trisb=0b00000000; //tudo sadas
   porta=0;//zera tudo
   portb=0;
   option_reg=0x80; //
   intcon=0b11100000; //liga interrupes gerais e perifricos e tmr0
   prog_mode();//testa boto para modo de programao



/*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/


   for(;;){

     le_adc();//atualiza display com leitura do potencimetro


     if(btn==0){ //testa boto de insero de senha
      led=1;
      timer5=TEMPO_PROG_MAX; //reseta tempo de programao
      timer6= TEMPO_ENTRAR_SENHA;//inicia tempo de time out
      val[ptn1]=le_adc();//le valor da posio atual do potencimetro e salva no array

      h=ptn1;//pega posio atual do ponteiro
      ptn1++;//incrementa ponteiro do array
      if(ledprog==1){//se em modo de programao ento grava digitos na eeprom
        write_eeprom(h,val[h]);//salva na eeprom
        if(ptn1>=QUANT_DIGITOS){//verifica se foi o ltimo nmero da senha
        ledprog=0;//apaga led de programao
        led=0;//apaga led de inicio de insero de senha
        ptn1=0;//zera ponteiro do array
        }
      }
      else{//se no est em programao est ento em modo de execuo
        if(ptn1>=QUANT_DIGITOS){
          for(g=0;g<QUANT_DIGITOS;g++){//compara valor inserido com a eeprom
            if(val[g]==read_eeprom(g))continue;//se igual continua no loop
            ptn1=0;//senha errada, reseta ponteiro das senhas e
            led=0;//apaga led
            break;//sai do loop forado
          }//for
          if(g>=QUANT_DIGITOS) {
              led=0;
              out=1;//liga fechadura eltrica
              delay_ms(TEMPO_ACIONAMENTO);//da um tempo acionando a fechadura eltrica
              out=0;//desliga fechadura
              ptn1=0;//reseta ponteiro das senhas
            }
        }
      }//else

     // timer7=TEMPO_PARADA;
      while(btn==0)delay_ms(10);
      flagat=0;
     }// if(btn==0)




   time_out_insercao();//conta tempo mximo para inserir senha
   time_out_prog();//conta tempo mximo para programao
   }//for(;;)

}//void main()
