/************************************************************************************

    COFRE DE COMBINAO USANDO POTNCIOMETRO LINEAR DE 5K


    SEM BOTO DE INSERCAO MAS USANDO TEMPO DO POTENCIMETRO SEM SE MOVER

    C/ INDICAO POR 2 DISPLAYS DE 7 SEGUIMENTOS NODO COMUM (NMEROS DE 00 A 99)

    C/ 5 DGITOS DE SENHA


    USANDO O MICROCONTROLADOR PIC16F628A


    AUTOR: CLAUDIO LARIOS

    DATA:30/12/2023



    MODO PROGRAMAO: MANTENHA BOTO 'PROG' APERTADO E LIGUE A
         ALIMENTAO. LED 'PROG' ACENDER.
         ENTRE COM A SENHA:
         PARA INICIAR A INSERO VIRE O KNOB TODO PARA A POSIO ZERO. IR
         ACENDER O DISPLAY E O LED DE INSERO.
         IMEDIATAMENTE VIRE O KNOB AT A POSIO DESEJADA QUE SER INDICADA NO
         DISPLAY O SEU VALOR.
         AGUARDE SEM MOVIMENTAR O KNOB AT O LED PISCAR PARA INDICAR A INSERO.
         GIRE O KNOB AT A PRXIMA POSIO DESEJADA E REPITA O PROCESSO
         COM TODA A SENHA. AO ENTRAR A LTIMA POSIO
         O LED 'PROG' E OS DISPLAYS APAGARO, INDICANDO SUCESSO NA GRAVAO.

    USO: REPITA A SENHA GRAVADA DE 5 DGITOS DA MESMA FORMA QUE NA PROGRAMAO.
         LEMBRE-SE DE GIRAR O  KNOB TODO A ESQUERDA PARA INICIAR O PROCESSO.

    OBS: O BOTO 'SW_PROG' NO DEVER SER ACESSADO PELO LADO DE FORA DO COFRE, MAS
         UNICAMENTE PELO LADO DE DENTRO. SE NO PODERIA SER REESCRITO UMA
         NOVA SENHA POR QUALQUER PESSOA.

    USE COMPILADOR CCS V. 3.200

    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG PICSOURCE

*******************************************************************************/
#include <16F628A.h>
#use delay(clock=4000000)//USE CRISTAL EXTERNO DE 4 MHZ
#fuses NOWDT,HS, NOPUT, NOPROTECT, BROWNOUT, NOMCLR, NOLVP, NOCPD



//bits
#bit bprog    =  0x05.5 // pino 4
#bit ledprog  =  0x05.0 // pino 17
#bit led      =  0x05.1 // pino 18
#bit out      =  0x05.2 // pino 1
#bit rc       =  0x05.3 // pino 2
#bit rc_dir   =  0x85.3 // pino 2 direo
#bit sel      =  0x06.7//  pino 13 seleo de display

//bytes
#byte  trisa = 0x85
#byte  trisb = 0x86
#byte  porta   = 0x05
#byte  portb   = 0x06
#byte  option_reg = 0x81
#byte  intcon = 0x0b


#define VALOR_PEGA_NUM 100// valor para pegar a centsima leitura no adcrc para comparao com inicial
#define VALOR_ESTAB 5//25valor para o filtro de estabilizao de leitura
#define TEMPO_PARADA  3000 //carga para tempo p/ insero de dgito aps parar de rodar o potencimetro
#define VALOR_PARA_ZERAR 6// elimina leitura at este valor com o potencimetro no mnimo
#define TEMPO_ENTRAR_SENHA 30000 // carga para o tempo de entrar a senha
#define TEMPO_PROG_MAX 25000 // carga para o tempo de time out para a programao
#define QUANT_DIGITOS  5  //quantidade de digitos na senha = 5
#define TEMPO_ACIONAMENTO 5000 //tempo que fechadura eltrica  acionada (em ms)

//constantes para converso 7 seguimentos

//                         PGFEDCBA   SEGUIMENTOS
//                   PORTB 76543210
int8 const NUM[] = {     0B01000000,//0
                         0B01111001,//1
                         0B00100100,//2
                         0B00110000,//3
                         0B00011001,//4
                         0B00010010,//5
                         0B00000010,//6
                         0B01111000,//7
                         0B00000000,//8
                         0B00010000 //9
                                        };


//==============================================================================

//variveis globais
unsigned int1 btn=1;//flag btn age como se fosse um boto apertado (0=ligado 1= desligado)
unsigned int1 flagat=0;//flag auxiliar nos processos entre subrotinas
unsigned int1 flagv=0;//flag que indica qual display ser mostrado
unsigned int1 flagok=0;//flag sinaliza que a converso de um valor em 7 seg est disponvel para os displays
unsigned int1 flagrcok=0;//flag sinaliza que um valor medido do potencimetro est disponivel
unsigned int1 flagx=0;//flag de controle para obter amostras de leituras do pot com atrasos entre elas
unsigned int1 flagapaga=0;//flag de controle do display sendo 0=apagados e 1=acesos
unsigned int8 val[QUANT_DIGITOS];//array para salvar provisriamente os digitos da senha
unsigned int8 ptn1=0;//ponteiro do array acima
unsigned int8 contrc=0;//contador usado para obter valor do potencimetro pelo mtodo  RC
unsigned int8 oldb=50;//salva leitura anterior do potencimetro
unsigned int8 tmp=0,tmp1=0;//variveis usadas no filtro anti-flickering
unsigned int8 v1,v2,cnt2=0;//variveis usadas na rotina de deteco de parada de movimento do eixo do pot
unsigned int8 g,h;//uso geral
unsigned int8 contd=0;//contador usado na multiplexao dos displays
unsigned int8 dig0a=8,dig1a=8,dig0=8,dig1=8;//usados para guardar dados a serem mostrados nos displays
unsigned int16 timer7=0;//usado para contar o tempo de parada do movimento do potencimetro
unsigned int32 timer5=0;//usado para contar o tempo mximo em modo de programao e sair deste
unsigned int32  timer6=0;//usado para contar o tempo mximo em modo de insero de senha e sair deste

/*******************************************************************************
          SUB-ROTINAS
*******************************************************************************/
#INT_TIMER0
void INT_TIMER0_ISR(){

 //calcula posio do potencimetro pelo tempo de carga dos capacitores
  if(flagrcok==0){
    rc_dir=1;//entrada
    while(rc==0)contrc++;//aguarda pino 2 (entrada) ficar 1 lgico
    rc=0;//zera sada
    rc_dir=0;//torna pino 2 sada
    flagrcok=1;//sinaliza com 1 no flag que existe uma leitura do potencimetro
  }
  contd++;//contador auxiliar para multiplexao dos displays de 7 seguimento
  if(contd>=8){//mostra no display a cada 8 interrupes
   contd=0;
   portb=0xff; //desliga todos os seguimentos
   if(flagapaga){
   if(flagok){//se tem alterao no valor repassa para os registradores do displays
     dig0a=dig0;
     dig1a=dig1;
     flagok=0;//reseta flag
   }

   if(flagv){//verifica a vez de quem vai ser mostrado: dezena ou unidade.
     portb= NUM[dig1a]|0x80;
     flagv=0;//inverte flag
   }
   else{
    portb= NUM[dig0a];
    flagv=1;//inverte flag
   }
  }//if(flagapaga){
 }//if(contd>=8){
}//void INT_TIMER0_ISR(){


/*******************************************************************************
          le_adc()
*******************************************************************************/
//rotina de leitura do potencimetro e acondicionamento de valores
//mtodo de carga e descarga de capacitor
unsigned int8 le_adc(){
  unsigned int8 b,c;

   while(flagrcok==0);
   b=contrc;//pega valor da contagem do tempo rc
   contrc=0;//reseta contador
   flagrcok=0;//prepara para nova contagem na isr do timer 0

   flagok=0;//inibe troca de valores na isr do timer 0

   b=(b + oldb )/2;//mdia com leitura anterior

   //------------------------------------------------
   //------------filtro anti-flickering--------------------
   //Filtro digital para estabilizao da leitura no
   //display de 7 seguimentos

   if(b==oldb){tmp=0;tmp1=0;}

   else {if(b>oldb){
     tmp++;
     tmp1=0;
     if(tmp>=VALOR_ESTAB){oldb=b; tmp=0;}else {b=oldb;}
    }
     else {if(b<oldb){
       tmp1++;
       tmp=0;
       if(tmp1>=VALOR_ESTAB){oldb=b;tmp1=0;}else{b=oldb;}
     }
    }
   }
   //--------------------------------------------------------
   c=b;
   oldb=b;//salva valor para prxima comparao
   //Ajusta a faixa a ser mostrada no display  de 0 a 99
   if(c>=VALOR_PARA_ZERAR)//despreza valor no mnimo do potencimetro e zera
     c=c-VALOR_PARA_ZERAR;
   else c=0;

   if(c>=100)c=99;//limita escala

   b=c;
   dig1=0;
   //converte para decimal dgito das dezenas
   while (c>=10){c=c-10; dig1++;}
   //obtm a unidade com o resto
   dig0=c;


   flagok=1;//autoriza troca de valores para apresentao na isr do timer 0
   return(b);//retorna valor
}



/*******************************************************************************
          prog_mode()
*******************************************************************************/
//Modo de programao de senha pelo potencimetro e boto
void prog_mode(){

  if(!bprog){
    delay_ms(50);
    if(!bprog){
      ledprog=1;//acende led
      timer5=TEMPO_PROG_MAX;
    }
  }
}

/*******************************************************************************
          time_out_parado()
*******************************************************************************/
//aguarda tempo sem movimento do pot para inserir senha
void time_out_parado(void){
   if(timer7==0)return;
   if(flagat==0)return;
   timer7--;
   if(timer7==0){

     led=0;//pisca led para indicar que foi capturado dgito mostrado nos displays
     delay_ms(200);
     led=1;
     btn=0;//ativa flag para indicar que pode pegar valor dos displays como dgito da senha
     flagat=0;
   }//if(timer7==0){
}


/*******************************************************************************
          time_out_prog()
*******************************************************************************/
//sai do modo de programao por tempo
void time_out_prog(void){
  if(ledprog){
   timer5--;
   if(timer5==0){ledprog=0; flagapaga=0;}
  }
}
/*******************************************************************************
          time_out_abert()
*******************************************************************************/
//sai do modo de insero de senha por tempo
void time_out_insercao(void){
  if(led){
   timer6--;
   if(timer6==0){led=0; flagapaga=0; ptn1=0;delay_ms(500); timer6=1;}//xxxxx
  }
}

/*******************************************************************************
          inserindo_senha()
*******************************************************************************/
//rotina de insero de senha
//verifica se potencimetro se movimenta ou se parou de movimentar.
void inserindo_senha(void){

  if(!led){ flagat=0; return;}

  if(flagx==0){v1 = le_adc(); flagx=1;cnt2=VALOR_PEGA_NUM; return;}
  //necessita cnt2 para vencer o efeito do filtro anti_flickering
  //a leitura v1 est distante da leitura de v2 pelo valor carregado em cnt2
  //com isso se garante que possa ser observado as variaes de movimento do potencimetro
  if(cnt2){
    cnt2--;//decrementa
    if(cnt2==0){
      v2=le_adc();//l valor do potencimetro
      flagx=0;
    }else return;
  }

  if((v1 == v2) && (flagat==0)){timer7=TEMPO_PARADA; return;}// medies iniciais em '0'

  if(v1 > v2){flagat=1; timer7=TEMPO_PARADA;return;}//medies diferentes= pot movimentando
  if(v1 < v2){flagat=1; timer7=TEMPO_PARADA;return;}
  if(v1 == v2)time_out_parado();//pot parado ->decrementa contador tempo e ao final ativa flag btn=0
}

/*******************************************************************************
          insercao()
*******************************************************************************/
//verifica nicio da insero de senha
void inicio_insercao(void){
 if(led)return;
 if(le_adc()==0){//verifica se pot foi levado a zero
   led=1;// inicia a insero
   timer6= TEMPO_ENTRAR_SENHA;//carrega tempo mximo em modo de insero
   ptn1=0;
   flagat=0;//despreza leituras iniciais
   flagapaga=1;//sinaliza para mostrar digitos que estavam apagados
 }
}
/*******************************************************************************
          inserindo_senha()


/*******************************************************************************
          ROTINA MAIN
*******************************************************************************/

void main() {


   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);//desliga timer 1
   setup_comparator(NC_NC_NC_NC);//desliga comparadores
   setup_vref(FALSE);//desliga vref
   trisa=0b11111000;//ajusta entradas e sadas
   trisb=0b00000000; //tudo sadas
   porta=0;//zera tudo
   portb=0;
   option_reg=0x80; //
   intcon=0b11100000; //liga interrupes gerais e perifricos e tmr0
   prog_mode();//testa boto para modo de programao



/*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/


   for(;;){

     le_adc();//atualiza display com leitura do potencimetro

     inicio_insercao();//verifica insero de senha se potencimetro foi virado a esquerda =00
     inserindo_senha();//rotina de insero de senha: verifica se pot esta girando ou se parou


     if(btn==0){ //testa flag de insero de senha: se 0 captura a senha mostrada nos displays
      led=1;
      timer5=TEMPO_PROG_MAX; //reseta tempo de programao
      timer6= TEMPO_ENTRAR_SENHA;//inicia tempo de time out
      val[ptn1]=le_adc();//le valor da posio atual do potencimetro e salva no array

      h=ptn1;//pega posio atual do ponteiro
      ptn1++;//incrementa ponteiro do array
      if(ledprog==1){//se em modo de programao ento grava digitos na eeprom
        write_eeprom(h,val[h]);//salva na eeprom
        if(ptn1>=QUANT_DIGITOS){//verifica se foi o ltimo nmero da senha
        ledprog=0;//apaga led de programao
        led=0;//apaga led de inicio de insero de senha
        flagapaga=0;//apaga displays
        ptn1=0;//zera ponteiro do array
        }
      }
      else{//se no est em programao est ento em modo de execuo
        if(ptn1>=QUANT_DIGITOS){
          for(g=0;g<QUANT_DIGITOS;g++){//compara valor inserido com a eeprom
            if(val[g]==read_eeprom(g))continue;//se igual continua no loop
            ptn1=0;//senha errada, reseta ponteiro das senhas e
            for(h=0;h<5;h++){
            led=1;
            delay_ms(250);
            led=0;//pisca led para indicar erro
            delay_ms(250);
            flagapaga=0;//apaga displays
            }
            break;//sai do loop forado
          }//for
          if(g>=QUANT_DIGITOS) {
              led=0;
              flagapaga=0;//apaga displays
              out=1;//liga fechadura eltrica
              delay_ms(TEMPO_ACIONAMENTO);//da um tempo acionando a fechadura eltrica
              out=0;//desliga fechadura
              ptn1=0;//reseta ponteiro das senhas

            }
        }
      }//else

      timer7=TEMPO_PARADA;
      btn=1;
      flagat=0;
     }// if(btn==0)



   time_out_parado();//tempo sem movimento no pot
   time_out_insercao();//conta tempo mximo para inserir senha
   time_out_prog();//conta tempo mximo para programao
   }//for(;;)

}//void main()
